"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.factory = void 0;
/**
 * Factory function that creates the disc-union functions. Use this
 * to have a different default discriminant than 'type'
 * @param factoryTypeKey - The default discriminant property of the functions
 */
var factory = function (factoryTypeKey) {
    function match(value, handlers, ot, tk) {
        var typeKey;
        var otherwise;
        if (typeof ot === "function") {
            otherwise = ot;
            typeKey = tk !== null && tk !== void 0 ? tk : factoryTypeKey;
        }
        else {
            typeKey = ot !== null && ot !== void 0 ? ot : factoryTypeKey;
        }
        var handler = handlers[value[typeKey]];
        return handler ? handler(value) : otherwise(value);
    }
    var is = function (type, value, typeKey) {
        if (typeKey === void 0) { typeKey = factoryTypeKey; }
        return value[typeKey] === type;
    };
    function discUnion(constructors, typeKey, prefix) {
        var _a, _b;
        if (typeKey === void 0) { typeKey = factoryTypeKey; }
        if (prefix === void 0) { prefix = ''; }
        var constructorsWithType = {};
        var _loop_1 = function (key) {
            var fullKey = prefix + key;
            constructorsWithType[key] = attachExtras(function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                return createType(fullKey, constructors[key].apply(constructors, args), typeKey);
            }, fullKey, typeKey);
        };
        for (var key in constructors) {
            _loop_1(key);
        }
        var symbols = (_b = (_a = Object.getOwnPropertySymbols) === null || _a === void 0 ? void 0 : _a.call(Object, constructors)) !== null && _b !== void 0 ? _b : [];
        symbols.forEach(function (symbol) {
            constructorsWithType[symbol] = attachExtras(function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                return createType(symbol, constructors[symbol].apply(constructors, args), typeKey);
            }, symbol, typeKey);
        });
        return constructorsWithType;
    }
    ;
    var createType = function (type, obj, typeKey) {
        var _a;
        if (typeKey === void 0) { typeKey = factoryTypeKey; }
        return (__assign(__assign({}, obj), (_a = {}, _a[typeKey] = type, _a)));
    };
    var validate = function (type, obj, typeKey) {
        if (typeKey === void 0) { typeKey = factoryTypeKey; }
        if (is(type, obj, typeKey))
            return obj;
        else
            throw new Error(formatError(type.toString(), obj[typeKey].toString()));
    };
    function get(type, obj, typeKey) {
        if (typeKey === void 0) { typeKey = factoryTypeKey; }
        if (is(type, obj, typeKey))
            return obj;
        else
            return null;
    }
    var map = function (type, obj, mapper, typeKey) {
        if (typeKey === void 0) { typeKey = factoryTypeKey; }
        if (is(type, obj, typeKey))
            return mapper(obj);
        else
            return obj;
    };
    var attachExtras = function (fn, type, typeKey) {
        if (typeKey === void 0) { typeKey = factoryTypeKey; }
        var extras = {
            key: type,
            is: function (obj) {
                return obj[typeKey] === type;
            },
            map: function (obj, mapper) {
                return extras.is(obj) ? mapper(obj) : obj;
            },
            get: function (obj) {
                return extras.is(obj) ? obj : null;
            },
            validate: function (obj) {
                if (extras.is(obj))
                    return obj;
                else
                    throw new Error(formatError(type.toString(), obj[typeKey].toString()));
            },
        };
        return Object.assign(fn, extras);
    };
    return {
        discUnion: discUnion,
        match: match,
        map: map,
        get: get,
        is: is,
        validate: validate,
        createType: createType,
        attachExtras: attachExtras
    };
};
exports.factory = factory;
