import { factory } from './factory';
export * from './types';
export { factory };
/**
 * Accepts an object of constructor functions,
 * and wraps those functions to add a type property
 * to their result
 *
 * @function
 * @param constructors - Object containing constructors
 * @param typeKey - Discriminant property
 */
export declare const discUnion: <T extends import("./types").Constructors, P extends string = "", TypeKey extends import("./types").ObjKey = "type">(constructors: T, typeKey?: TypeKey, prefix?: P) => import("./types").ConstructorsWithType<T, TypeKey, P>;
/**
 * Returns the result of mapper if value is of the specified type,
 * otherwise returns the value unchanged
 *
 * @function
 * @param type - Type to match
 * @param obj - Value to map over
 * @param mapper - Function to be called on obj
 * @param typeKey - Discriminant property
 */
export declare const map: <T extends import("./types").DiscUnionType<TypeKey>, K extends T[TypeKey], R, TypeKey extends import("./types").ObjKey = "type">(type: K, obj: T, mapper: (o: Extract<T, { [M in TypeKey]: K; }>) => R, typeKey?: TypeKey) => T | R;
/**
 * Takes an object to match, and an object of handlers whose
 * keys are the possible types of the discriminated union.
 *
 * Exhaustive unless an "otherwise" handler is passed in as a third argument,
 * in which case the handler object may be partial.
 *
 * @function
 * @param obj - Object to match
 * @param handlers - Object whose keys are possible types of the discriminated union, and whose values are handlers for those types
 * @param otherwise (optional) - Handler if no match is found
 * @param typeKey (optional) - Discriminant property
 * @returns Result of the matched handler, or the result of the "otherwise" handler
 */
export declare const match: {
    <T extends import("./types").DiscUnionType<TypeKey>, R extends import("./types").Handlers<T, unknown, TypeKey>, TypeKey extends import("./types").ObjKey = "type">(value: T, handlers: import("./types").Handlers<T, unknown, TypeKey> extends R ? R : import("./types").Handlers<T, unknown, TypeKey>, typeKey?: TypeKey | undefined): R extends import("./types").Handlers<T, infer RT, TypeKey> ? RT : never;
    <T_1 extends import("./types").DiscUnionType<TypeKey_1>, R_1 extends Partial<import("./types").Handlers<T_1, unknown, TypeKey_1>>, E, TypeKey_1 extends import("./types").ObjKey = "type">(value: T_1, handlers: import("./types").Handlers<T_1, unknown, TypeKey_1> extends R_1 ? R_1 : Partial<import("./types").Handlers<T_1, unknown, TypeKey_1>>, otherwise: (value: Exclude<T_1, { [M in TypeKey_1]: keyof R_1; }>) => E, typeKey?: TypeKey_1 | undefined): E | (R_1 extends Partial<import("./types").Handlers<T_1, infer RT_1, TypeKey_1>> ? RT_1 : never);
};
/**
 * Narrows the type of the value if it matches the specified type,
 * otherwise returns null
 *
 * @function
 * @param type - Type to match
 * @param obj - Value to get
 * @param typeKey - Discriminant property
 */
export declare const get: {
    <T extends import("./types").DiscUnionType<TypeKey>, K extends T[TypeKey], TypeKey extends import("./types").ObjKey = "type">(type: K, obj: Extract<T, { [M in TypeKey]: K; }>, typeKey?: TypeKey | undefined): Extract<T, { [M in TypeKey]: K; }>;
    <T_1 extends import("./types").DiscUnionType<TypeKey_1>, K_1 extends T_1[TypeKey_1], TypeKey_1 extends import("./types").ObjKey = "type">(type: K_1, obj: T_1, typeKey?: TypeKey_1 | undefined): Extract<T_1, { [M_1 in TypeKey_1]: K_1; }> | null;
};
/**
 * Determines if the value matches the specified type
 *
 * @function
 * @param type - Type to match
 * @param obj - Value to check
 * @param typeKey - Discriminant property
 */
export declare const is: <T extends import("./types").DiscUnionType<TypeKey>, K extends T[TypeKey], TypeKey extends import("./types").ObjKey = "type">(type: K, value: T, typeKey?: TypeKey) => value is Extract<T, { [M in TypeKey]: K; }>;
/**
 * Narrows the type of the value if it matches, throws
 * an error otherwise
 *
 * @function
 * @param type - Type to match
 * @param obj - Value to check
 * @param typeKey - Discriminant property
 */
export declare const validate: <T extends import("./types").DiscUnionType<TypeKey>, K extends T[TypeKey], TypeKey extends import("./types").ObjKey = "type">(type: K, obj: T, typeKey?: TypeKey) => Extract<T, { [M in TypeKey]: K; }>;
/**
 * Convenience function for adding a type discriminant
 * to an object
 *
 * @function
 * @param type - Type to create
 * @param obj - Value to use
 * @param typeKey - Discriminant property
 */
export declare const createType: <T extends import("./types").ObjKey, O extends object, TypeKey extends import("./types").ObjKey = "type">(type: T, obj: O, typeKey?: TypeKey) => import("./types").WithType<O, T, TypeKey>;
/**
 * Attaches "static" properties and methods to constructor function
 *
 * @function
 * @param fn - Constructor
 * @param type - Type string
 * @param typeKey - Discriminant property
 */
export declare const attachExtras: <Fn extends (args: unknown[]) => import("./types").DiscUnionType<TypeKey>, K extends import("./types").ObjKey, TypeKey extends import("./types").ObjKey = "type">(fn: Fn, type: K, typeKey?: TypeKey) => Fn & import("./types").ConstructorExtras<K, TypeKey>;
